package modelo;

import java.util.*;

public class Controlador {
  private Agenda agenda;

  private LinkedList oyentesModelo;

  public Controlador() {
    oyentesModelo = new LinkedList();
    agenda = new Agenda();
  }

  //Mtodos para que se registren y eliminen los oyentes del Modelo ----------

  public void addAgendaListener(AgendaListener listener){
    oyentesModelo.add(listener);
  }

  public void removeAgendaListener(AgendaListener listener){
    oyentesModelo.remove(listener);
  }

  //---------------------------------------------------------------------------

  // Interface para la gestin de notificaciones

  private interface Accion {

    void accion(AgendaListener listener);
  }

  private void notifica(Accion a) {

    Iterator it = oyentesModelo.iterator();
    while (it.hasNext()) {
      a.accion((AgendaListener) it.next());
    }

  }


  /**** Eventos del usuario ***/


  public void nuevoContacto(String nombre, String tfno, String foto) {
    Contacto c = new Contacto(nombre, tfno, foto);

    //Actualizar el modelo
    agenda.aadir(c);

    //Informar a los oyentes del modelo que ha habido un  cambio
    final EventAgenda e = new EventAgenda(agenda, c);
    notifica(new Accion () {

      public void accion (AgendaListener listener) {
        listener.alta(e);
      }
    }
    );
  }

  public void eliminarContacto(Contacto c){

    //Actualizo el modelo
    agenda.eliminar(c);

    //Informar a los oyentes del modelo que ha habido un  cambio
    final EventAgenda e = new EventAgenda(agenda, c);

    notifica(new Accion () {

      public void accion (AgendaListener listener) {
        listener.baja(e);
      }
    }
    );

  }

}
