package vistas;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import modelo.*;

/**
 * La ventana va a estar pendiente de los cambios en el modelo para ir
 * actualizandose. Por tanto, tiene que implementar la interfaz AgengaListener
 * e implementar de manera adecuada el modo de reaccionar ante los eventos que
 * en esta interfaz se especifican.
 * La clase Controlador se encargar de notificar estos eventos a la GUI.
 */


public class GUIAgenda extends JFrame implements AgendaListener{

  //La ventana mantiene una referencia al controlador de la aplicacin
  Controlador controlador;

  JPanel contentPane;
  BorderLayout borderLayout1 = new BorderLayout();

  private JScrollPane jScrollPane1 = new JScrollPane();

  //Una lista tiene asociado un modelo de lista
  //Las inserciones y eliminaciones no se hacen directamente sobre la JList
  //sino a travs de el modelo
  private JList jListContactos = new JList(new DefaultListModel());

  JButton jButtonNuevo = new JButton();
  JButton jButtonEliminar = new JButton();
  JToolBar jToolBarBotones = new JToolBar();


  //Construir el marco
  public GUIAgenda(Controlador controlador) {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {

      this.controlador = controlador;
      //Nos registramos como oyentes de los eventos de la agenda
      controlador.addAgendaListener(this);

      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  //Inicializacin de componentes
  private void jbInit() throws Exception  {

    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(borderLayout1);
    this.setSize(new Dimension(200, 300));
    this.setTitle("Agenda");
    this.setDefaultCloseOperation(EXIT_ON_CLOSE);

    // Establece seleccin simple de elementos en la lsita
    jListContactos.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

    jListContactos.addMouseListener(new java.awt.event.MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        jListContactos_mouseClicked(e);
      }
    });

    jButtonNuevo.setText("Nuevo");
    jButtonNuevo.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButtonNuevo_actionPerformed(e);
      }
    });

    jButtonEliminar.setText("Eliminar");
    jButtonEliminar.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButtonEliminar_actionPerformed(e);
      }
    });

    jScrollPane1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    jScrollPane1.getViewport().add(jListContactos, null);

    contentPane.add(jScrollPane1,  BorderLayout.CENTER);
    contentPane.add(jToolBarBotones,  BorderLayout.NORTH);
    jToolBarBotones.add(jButtonNuevo, null);
    jToolBarBotones.add(jButtonEliminar, null);

  }

  //Modificado para poder salir cuando se cierra la ventana
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      System.exit(0);
    }
  }

/*** Implementacin AgendaListener  *****/

  //Implementamos cmo reaccionar la ventana ante los eventos de la agenda
  public void alta (EventAgenda e){

    //Actualiza los elementos de la JList aadindolo al modelo
    ((DefaultListModel)jListContactos.getModel()).addElement(e.getContacto());

  }

  public void baja (EventAgenda e){

   //Actualiza los elementos de la JList aadindolo al modelo
   ((DefaultListModel)jListContactos.getModel()).removeElement(e.getContacto());

  }

  //------------------------------------------------------------------------

/*** Eventos de la interfaz ****/

  void jListContactos_mouseClicked(MouseEvent e) {

    if ((jListContactos.getModel().getSize() > jListContactos.getSelectedIndex()) && (jListContactos.getSelectedIndex() >= 0)) {

      Contacto contactoSeleccionado = (Contacto) jListContactos.getSelectedValue();

      //Quiero mostrar la ventana con los datos
      if (e.getClickCount() == 2) {
        DialogoAlta da = new DialogoAlta(controlador, contactoSeleccionado);
        da.setVisible(true);
      }
    }
  }


  void jButtonNuevo_actionPerformed(ActionEvent e) {
    DialogoAlta vtnaAlta = new DialogoAlta(controlador);
    vtnaAlta.setVisible(true);
  }

  void jButtonEliminar_actionPerformed(ActionEvent e) {

    if ((jListContactos.getModel().getSize() > jListContactos.getSelectedIndex()) && (jListContactos.getSelectedIndex() >= 0)) {

      Contacto contactoSeleccionado = (Contacto) jListContactos.getSelectedValue();

      //Eliminamos el contacto seleccionado
      controlador.eliminarContacto(contactoSeleccionado);
    }
  }

}
